------ Variables ------
local M = {}
local modAttached = false
local override = false

local couplerCache = {}
local couplerTags = {}

------ Functions ------

local function has_value(nTable, nValue)
    for index, value in pairs(nTable) do
        if value == nValue then
            return true
        end
    end
    return false
end

local function CloseParts()

    if string.find(tostring(v.data.options.partName), "wigeon") then
        electrics.values['doorlatch_L'] = 0
        electrics.values['doorlatch_R'] = 0
        electrics.values['canopy_latch'] = 0
    end


    electrics.values['hood'] = 0
    electrics.values['doorFL'] = 0
    electrics.values['doorFR'] = 0
    electrics.values['doorRL'] = 0
    electrics.values['doorRR'] = 0
end

local function getNodeID(name)
    local nodeID = nil
    for k, node in pairs(v.data.nodes) do
        if tostring(node.name) == tostring(name) then
            return node.cid
        end
    end
    return null
end

local function AdjustSpringStrength(node1, node2, strength)

    node1 = getNodeID(node1)
    node2 = getNodeID(node2)

    for k, beam in pairs(v.data.beams) do
        if string.find(tostring(beam.breakGroup), "_Openable") == nil then
            if (beam.id1 == node1 and beam.id2 == node2) or (beam.id2 == node1 and beam.id1 == node2) then

                beam.beamDamp = strength
                beam.beamSpring = strength

                obj:setBeamSpringDamp(beam.cid, beam.beamSpring, beam.beamDamp, -1, -1)

            end
        end
    end
end

local function BreakBeamFromNodes(node1, node2)
    for k, beam in pairs(v.data.beams) do
        if string.find(tostring(beam.breakGroup), "_Openable") == nil then
            if beam.id1 == node1 and beam.id2 == node2 then
                obj:breakBeam(k)
            elseif beam.id2 == node1 and beam.id1 == node2 then
                obj:breakBeam(k)
            end
        end
    end
end

local function removeNodeCollision(node)
    local NORMALTYPE = 0
    local NODE_FIXED = 1

    local ntype = NORMALTYPE
    if node.fixed == true then
        ntype = NODE_FIXED
    end

    local collision
    if node.collision ~= nil then
        collision = node.collision
    else
        collision = true
    end

    local selfCollision
    if node.selfCollision ~= nil then
        selfCollision = node.selfCollision
    else
        selfCollision = false
    end

    local staticCollision
    if node.staticCollision ~= nil then
        staticCollision = node.staticCollision
    else
        staticCollision = true
    end

    local frictionCoef = type(node.frictionCoef) == 'number' and node.frictionCoef or 1
    local slidingFrictionCoef = type(node.slidingFrictionCoef) == 'number' and node.slidingFrictionCoef or frictionCoef
    local noLoadCoef = type(node.noLoadCoef) == 'number' and node.noLoadCoef or 1
    local fullLoadCoef = type(node.fullLoadCoef) == 'number' and node.fullLoadCoef or 0
    local loadSensitivitySlope = type(node.loadSensitivitySlope) == 'number' and node.loadSensitivitySlope or 0

    local nodeWeight = node.nodeWeight or 0

    local nodeMaterialTypeID
    if node.nodeMaterial ~= nil then
        nodeMaterialTypeID = node.nodeMaterial
        if type(nodeMaterialTypeID) ~= "number" then
            log_jbeam('D', "jbeam.pushToPhysics", "invalid node material id:" .. tostring(nodeMaterialTypeID))
            nodeMaterialTypeID = 0
        end
    else
        nodeMaterialTypeID = 0
    end

    local pos = vec3(obj:getNodePosition(node.cid))
    obj:setNode(node.cid, pos.x, pos.y, pos.z, nodeWeight, ntype, frictionCoef, slidingFrictionCoef,
        node.stribeckExponent or 1.75, node.stribeckVelMult or 1, noLoadCoef, fullLoadCoef, loadSensitivitySlope,
        node.softnessCoef or 0.5, node.treadCoef or 0.5, node.tag or '', node.couplerStrength or math.huge,
        node.firstGroup or -1, false, collision, staticCollision, nodeMaterialTypeID)
end

local function breakConnectingBeams() -- searches for and breaks beams
    CloseParts()
    for k, beam in pairs(v.data.beams) do -- loop for each beam 
        if (string.find(tostring(beam.breakGroup), "_Openable") ~= nil) then
            modAttached = true
        end

        if beam.breakGroup ~= nil and beam.breakGroup ~= "" then -- if beam contains a breakGroup
            ------ All vehicles ------
            if string.find(tostring(beam.breakGroup), "hoodlatch") ~= nil or -- if beam is for the hood
                string.find(tostring(beam.breakGroup), "tailgatelatch") ~= nil or -- or if beam is for the tailgate

                string.find(tostring(beam.breakGroup), "doorlatch") ~= nil then -- or if beam is for a door
                if string.find(tostring(beam.breakGroup), "_Openable") == nil then -- if beam hasnt be added by this mod
                    if (modAttached or override) then
                        obj:breakBeam(k) -- breaks beam
                    end
                end
            end

            ------ Wendover ------
            if string.find(tostring(v.data.options.partName), "wendover") then
                if string.find(tostring(beam.breakGroupType), "1") ~= nil and
                    string.find(tostring(beam.breakGroup), "hoodhinge") then
                    if string.find(tostring(beam.breakGroup), "_Openable") == nil then -- if beam hasnt be added by this mod
                        if (modAttached or override) then
                            obj:breakBeam(k) -- breaks beam
                        end
                    end
                end
                if string.find(tostring(beam.beamType), "BOUNDED") ~= nil and
                    string.find(tostring(beam.breakGroup), "hoodhinge") then
                    if string.find(tostring(beam.breakGroup), "_Openable") == nil then -- if beam hasnt be added by this mod
                        if (modAttached or override) then
                            obj:breakBeam(k) -- breaks beam
                        end
                    end
                end
            end

            ------ Vivace ------
            if string.find(tostring(v.data.options.partName), "vivace") then
                if string.find(tostring(beam.breakGroup), "latch") ~= nil then
                    -- string.find(tostring(beam.breakGroup), "tailgatehinge") ~= nil then
                    if string.find(tostring(beam.breakGroup), "_Openable") == nil then
                        if (modAttached or override) then
                            obj:breakBeam(k)
                        end
                    end
                end
            end
        end
    end

    if (modAttached) then
        local n = {}

        ------ etkc ------
        if string.find(tostring(v.data.options.partName), "etkc") then
            ------ Disable node self colisions ------
            ------ FL Door ------
            n[0] = getNodeID('d4l')
            n[1] = getNodeID('d7l')
            ------ FR Door ------
            n[3] = getNodeID('d4r')
            n[4] = getNodeID('d7r')

            ------ prevent windows from breaking------
            ------ FL Door ------
            AdjustSpringStrength('d7l', 'd8l', 0)
            AdjustSpringStrength('d7l', 'd9l', 0)
            AdjustSpringStrength('d7l', 'd10l', 0)
            AdjustSpringStrength('d7l', 'd11l', 0)
            AdjustSpringStrength('d7l', 'd12l', 0)
            AdjustSpringStrength('d7l', 'd13l', 0)
            ------ FR Door ------
            AdjustSpringStrength('d7r', 'd8r', 0)
            AdjustSpringStrength('d7r', 'd9r', 0)
            AdjustSpringStrength('d7r', 'd10r', 0)
            AdjustSpringStrength('d7r', 'd11r', 0)
            AdjustSpringStrength('d7r', 'd12r', 0)
            AdjustSpringStrength('d7r', 'd13r', 0)

            ------ Breaking extra beams (beams which either restrict opening or break anyway) ------
            BreakBeamFromNodes(getNodeID('h4l'), getNodeID('he3l'))
            BreakBeamFromNodes(getNodeID('h4r'), getNodeID('he3r'))
            BreakBeamFromNodes(getNodeID('p11l'), getNodeID('t3ll'))
            BreakBeamFromNodes(getNodeID('p11r'), getNodeID('t3rr'))
        end

        ------ Removing node self colisions ------
        for k, node in pairs(v.data.nodes) do
            if has_value(n, k) then
                removeNodeCollision(node)
            end
        end
    end
end

function OpenPart(node1, node2, originalSpring, openValue)
    node1 = getNodeID(node1)
    node2 = getNodeID(node2)

    if originalSpring == 0 then
        originalSpring = 2000000
    end

    for k, beam in pairs(v.data.beams) do
        if beam.beamPrecompression == 1 and string.find(tostring(beam.breakGroup), "_Openable") ~= nil then
            if (beam.id1 == node1 and beam.id2 == node2) or (beam.id2 == node1 and beam.id1 == node2) then

                if openValue ~= 0 then
                    beam.beamSpring = 0
                    beam.beamDamp = 0
                else
                    beam.beamSpring = originalSpring
                    beam.beamDamp = 10
                end

                obj:setBeamSpringDamp(beam.cid, beam.beamSpring, beam.beamDamp, -1, -1)

            end
        end
    end
end

M.onReset = breakConnectingBeams
M.onInit = CloseParts


return M
